(:~
 : This module offers a set of functions that are necessary for creating
 : the edition nav-bar of the digital edition.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 :)
xquery version "3.1";

module namespace nav-utils="http://bdn-edition.de/ns/nav-utils";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
(:declare namespace templates="http://exist-db.org/xquery/templates";:)
declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace xhtml="http://www.w3.org/1999/xhtml";


import module namespace templates="http://exist-db.org/xquery/templates";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "utils.xqm";
import module namespace whitelist="https://bdn-edition.de/ns/whitelist" at "whitelist.xqm";
import module namespace xml-utils="http://bdn-edition.de/portal/xml-utils" at "xml-utils.xqm";

declare function nav-utils:nav-injection($node as node(), $model as map(*))
as element(nav)
{
  <nav id="edition-menu" class="nav navbar navbar-default">
      {nav-utils:make-xs-sm-navbar($model)}

      <div class="row hidden-xs hidden-sm">
          <div class="dropdown-container">
              <span class="dropdown edition-menu-button">
                  <a class="btn dropdown-toggle edition-menu-link" href="#" role="button" id="view-button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                      <i class="far fa-eye"></i>
                      Ansichten <i class="fas fa-caret-down"></i>
                  </a>

                  <div class="dropdown-menu edition-menu-dropdown" aria-labelledby="dropdownMenuLink">
                      {templates:process(<span data-template="nav-utils:make-views-content" />, $model)}
                  </div>
              </span>

              {if (not(local:in-split-xml())) then
                  ()
              else
                  <span class="dropdown edition-menu-button">
                      <a class="btn dropdown-toggle edition-menu-link" href="#" role="button" id="overview-button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                          <i class="fas fa-map-marked"></i>
                          Bandübersicht <i class="fas fa-caret-down"></i>
                      </a>

                      <div class="dropdown-menu edition-menu-dropdown" aria-labelledby="dropdownMenuLink">
                          {templates:process(<span data-template="nav-utils:make-work-overview" />, $model)}
                      </div>
                  </span>
              }

              <span class="dropdown edition-menu-button">
                  <a class="btn dropdown-toggle edition-menu-link" href="#" role="button" id="key-button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                      <i class="fa fa-key" />
                      Legende <i class="fas fa-caret-down"></i>
                  </a>

                  <div class="dropdown-menu edition-menu-dropdown" aria-labelledby="dropdownMenuLink">
                      {templates:process(<span data-template="nav-utils:make-key-content" />, $model)}
                  </div>
              </span>


              {if (local:in-split-xml()) then
                  ()
              else
                  <span class="dropdown edition-menu-button">
                      <a class="btn dropdown-toggle edition-menu-link" href="#" role="button" id="metadata-button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                          <i class="fas fa-code"></i>
                          Metadaten <i class="fas fa-caret-down"></i>
                      </a>

                      <div class="dropdown-menu edition-menu-dropdown" aria-labelledby="dropdownMenuLink">
                          {templates:process(<span data-template="nav-utils:make-metadata" />, $model)}
                      </div>
                  </span>
              }

              {if (not(local:in-split-xml())) then
                  ()
              else
                  <span class="dropdown edition-menu-button">
                      <a class="btn dropdown-toggle edition-menu-link" href="#" role="button" id="index-button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                          <i class="fa fa-puzzle-piece" aria-hidden="true"> </i>
                          Register <i class="fas fa-caret-down"></i>
                      </a>

                      <div class="dropdown-menu edition-menu-dropdown" aria-labelledby="dropdownMenuLink">
                          {
                          let $file := request:get-parameter("file", "")
                          return nav-utils:inject-indices($file)
                          }
                      </div>

                  </span>
              }
          </div>
          <div class="title-display" aria-labelledby="titleDisplay">
              {templates:process(<span data-template="bdn-utils:get-title" />, $model)}
          </div>
      </div>
  </nav>
};


(:~
 : Creates the content for the key panel.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @param $model The model of the current page
 : @return element(div) Containing the body content
 :)
declare function nav-utils:make-key-content($node as node()*, $model as map(*))
as element(div)*
{
  let $filename := request:get-parameter("file","")
  let $doc := nav-utils:get-file($filename)

  let $editions := string-join($doc//tei:witness/@xml:id/string(), " ")
  let $editions := distinct-values(tokenize($editions, "[#\s]+"))
  let $edition := request:get-parameter("edition","digital")
  let $sigla := nav-utils:make-key-sigla()

  return
    <div class="key-container">
      <div class="keyEdtNav">Textquellen <i class="fa fa-caret-up"></i></div>
      <span class="keyEdtContent">
        <table style="margin-left: 20px; margin-top: 5px; margin-bottom: 10px;">
        {
        for $edition in $editions
          where $edition != ""
          let $description := $doc//tei:witness[@xml:id = $edition]/tei:label/string()
          return
            <tr>
              <td>{$edition}: </td>
              <td> {$description} </td>
            </tr>
        }
        </table>
      </span>

      <div class="keySignNav">Textkritische Siglen <i class="fa fa-caret-up"></i></div>
      <span class="keySignContent">
        <table style="margin-left: 20px; margin-top: 5px; margin-bottom: 10px;">
        {
        if ($edition = 'digital')
          then ($sigla("all"), $sigla("digital"), $sigla("critical-digital"))
        else if ($edition = 'app')
          then ($sigla("all"), $sigla("critical"), $sigla("critical-digital"))
        else
          $sigla("all")
        }
        </table>
      </span>
      <div class="keySymbolsNav">Symbole u. Ä. <i class="fa fa-caret-up"></i></div>
      <span class="keySymbolsContent">
        <table style="margin-left: 20px; margin-top: 5px; margin-bottom: 10px;">
        {
        if ($edition = 'digital')
          then ($sigla("single-digital"), $sigla("digital-symbols"))
        else if (contains($edition, $editions))
          then $sigla("single-digital")
        else ()
        }
        </table>
      </span>
    </div>
};


(:~
 : Creates the sigla for the key panel.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @return map() Containing information about which sigla belong to which
 : edition view
 :)
declare function nav-utils:make-key-sigla() as map(*)
{
  let $all :=
    <span>
        <tr>
            <td>|a165|</td>
            <td>Seitenwechsel in a</td>
        </tr>
        <tr>
            <td>[...]</td>
            <td>Editorische Einfügung</td>
        </tr>
    </span>

  let $digital :=
    <span>
        <tr>
            <td><sup>a</sup>√</td>
            <td>Einfügung in a</td>
        </tr>
        <tr>
            <td>a: <em>Ø</em></td>
            <td>Auslassung in a</td>
        </tr>
    </span>

  let $digital-symbols :=
    <span>
        <tr>
            <td><i class="fa fa-book" style="cursor:default" /></td>
            <td>Bibliographischer Nachweis</td>
        </tr>
        <tr>
            <td><i class="fa fa-user" style="cursor:default" /></td>
            <td>Informationen zur genannten Person</td>
        </tr>
        <tr>
            <td><i class="far fa-comment" style="cursor:default" /></td>
            <td>Editorische Erläuterung</td>
        </tr>
        <tr>
            <td><span class="expan-key">u. s. w.</span></td>
            <td>Abkürzungsauflösung</td>
        </tr>
    </span>


  let $single-digital :=
    <span>
        <tr>
            <td><i class="fa fa-wrench" style="cursor:default" /></td>
            <td>Satzfehlerkorrektur</td>
        </tr>
        <tr>
            <td><i class="fas fa-pen-fancy" style="cursor:default" /></td>
            <td>Autorschaftliche Korrektur</td>
        </tr>
        <tr>
            <td><i class="fa fa-external-link-alt" style="cursor:default" /></td>
            <td>Verlinkung innerhalb des Werks</td>
        </tr>
        <tr>
            <td><i class="fas fa-quote-right" style="cursor:default" /></td>
            <td>Zitat eines anderen Textteils</td>
        </tr>
    </span>

  let $critical :=
    <span>
        <tr>
            <td><sup>/a</sup>...<sup>a\</sup></td>
            <td>Auslassung in a</td>
        </tr>
        <tr>
            <td>∥<sup>a</sup></td>
            <td>Einfügung in a</td>
        </tr>
        <tr>
            <td><sup>a</sup>...<sup>a</sup></td>
            <td>Längere Einfügung in a</td>
        </tr>
        <tr>
            <td>...<sup>a1</sup></td>
            <td>Variante in a</td>
        </tr>
    </span>

  let $critical-digital :=
    <span>
        <tr>
            <td>⌇<sup>a</sup></td>
            <td>Neue Zeile in a</td>
        </tr>
        <tr>
            <td>⌇⌇<sup>a</sup></td>
            <td>Neuer Absatz in a</td>
        </tr>
        <tr>
            <td><s>⌇</s><sup>a</sup></td>
            <td>Fehlender Zeilenumbruch in a</td>
        </tr>
        <tr>
            <td><s>⌇⌇</s><sup>a</sup></td>
            <td>Fehlender Absatzumbruch in a</td>
        </tr>
    </span>

  return
    map
    {
      "all": $all,
      "digital": $digital,
      "digital-symbols": $digital-symbols,
      "single-digital": $single-digital,
      "critical": $critical,
      "critical-digital": $critical-digital
    }
};

(:~
 : Creates the content for the views panel.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @param $model The model of the current page
 : @return element(span) Containing the body content
 :)
declare function nav-utils:make-views-content($node as node()*, $model as map(*))
as element(div)*
{
  let $filename := request:get-parameter("file","")
  let $file := nav-utils:get-file($filename)

  let $whitelist := doc($config:aux || "whitelist.xml")

  let $edts-tmp := string-join($file//tei:witness/@xml:id/string(), " ")
  let $editions := distinct-values(tokenize($edts-tmp, "[#\s]+"))
  (: makes iterating easier :)
  let $desc := string-join($file//tei:witness/tei:label, "_")
  let $baseText := $file//tei:witness[@n = 'base-text']/@xml:id/string()

  return
      <div class="views">
        <div class="dropdown-divider"></div>
        {
            if (not(matches($filename, "bahrdt-semler"))) then
                (<a class="dropdown-item" href="{nav-utils:make-url($filename, "digital", false())}">
                    Interaktive textkritische Ansicht mit {$baseText} als Leittext
                </a>,
                <div class="dropdown-divider"></div>,
                <a class="dropdown-item" href="{nav-utils:make-url($filename, "app", false())}">
                    Textkritische Ansicht mit {$baseText} als Leittext
                </a>,
                <div class="dropdown-divider"></div>,
                <span>
                    {
                    for $edition at $countEdt in $editions
                        for $date at $countDate in tokenize($desc, '_')
                            where $edition != "" and $countEdt = $countDate
                            return
                                if ($whitelist//*[@source = $filename
                                and @omitting-printing = $edition]) then
                                    <a class="dropdown-item disabled" href="{nav-utils:make-url($filename, $edition, false())}">
                                    Auflage {$edition} ({$date}) <em>[kein Text für diese Auflage vorhanden]</em>
                                    </a>
                                else
                                    <a class="dropdown-item" href="{nav-utils:make-url($filename, $edition, false())}">
                                    Auflage {$edition} ({$date})
                                    </a>
                    }
                </span>
                )
            else
            <a class="dropdown-item" href="{nav-utils:make-url($filename, "digital", false())}">
                Interaktive textkritische Ansicht
            </a>
        }
        <div class="dropdown-divider"></div>
        <a class="dropdown-item" href="{nav-utils:make-url($filename, "code", false())}">XML/TEI</a>
      </div>
};


(:~
 : Creates links to book.html or foreword.html with all necessary information,
 : i.e. file name and edition view.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @param $filename The current file
 : @param $edition The current edition view
 : @param $foreword true() if we want to set a link to the editorial foreword
 : @return The link as xs:string
 :)
declare function nav-utils:make-url($filename as xs:string, $edition as xs:string,
$foreword as xs:boolean) as xs:string*
{
    if ($foreword) then
        let $author :=
            if (matches($filename, "bahrdt-semler")) then
                "bahrdt-semler"
            else if (matches($filename, "-")) then
                substring-before($filename, "-")
            else
                ()
        return
            if ($author) then
                "foreword.html?author=" || $author
            else
                ()
    else
        let $url :=
            $config:app-root || "/book.html?"
            => substring-after("sade")
        return
            "book.html?&#38;file=" || $filename || "&#38;edition=" || $edition
};


(:~
 : Returns an opened XML file depending on the current file.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @param $file The current file
 : @return An opened file
 :)
declare function nav-utils:get-file($file as xs:string) as node()?
{
  let $file := if (contains($file, "noesselt")) then
    "noesselt-full.xml"
  else if (contains($file, "griesbach")) then
    "griesbach-full.xml"
  else if (contains($file, "less")) then
    "less-full.xml"
  else
    $file || ".xml"
  return
    if (contains($file, "full")) then
        doc($config:full-xmls || $file)
     else
        doc($config:sade-data || $file)
};


(:~ Returns a short bibliographic title needed for the collapsed navbar in the
 : edition views.  :)
declare function nav-utils:get-short-title($node as node(), $model as map(*))
{
  let $filename := request:get-parameter("file", "")
  let $author :=
      if (matches($filename, "-")) then
          bdn-utils:get-author-from-whitelist($filename)
        else
            ""
    return
        if ($author != "") then
          let $doc := doc($config:full-xmls || $author || "-full.xml")
          let $title := $doc//tei:title[@level = "a"]/tei:title[@type = "main"]/string()
          let $title := substring-before($title, "(")

          let $series-stmt := substring-before($title, ": ")

          return
            <span class="navbar-edition-label-text">{$series-stmt}: {$author}</span>
        else
            <span>{$filename}.xml</span>
};


declare function nav-utils:make-xs-sm-navbar($model as map(*))
as element(div)
{
   <div class="container-fluid visible-xs visible-sm">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <button type="button" class="navbar-toggle collapsed" data-toggle="collapse"
      data-target="#navbar-edition-burger" aria-expanded="false">
        <span class="sr-only">Toggle navigation</span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
      </button>
        <span class="visible-xs visible-sm navbar-edition-label navbar-brand">
            {templates:process(<span data-template="nav-utils:get-short-title" />, $model)}
        </span>
    </div>

    <!-- Collect the nav links, forms, and other content for toggling -->
    <div class="collapse navbar-collapse" id="navbar-edition-burger">
      <ul class="nav navbar-nav">
        <li class="dropdown">
          <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button"
          aria-haspopup="true" aria-expanded="false">Ansichten <span class="caret"></span></a>
          <ul id="navbar-edition-burger-views" class="dropdown-menu">
            {templates:process(<span data-template="nav-utils:make-views-content" />, $model)}
          </ul>
        </li>
        {
            if (not(local:in-split-xml())) then
                ()
            else
                <li class="dropdown">
                  <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button"
                  aria-haspopup="true" aria-expanded="false">Bandübersicht <span class="caret"></span></a>
                  <ul id="navbar-edition-burger-overview" class="dropdown-menu">
                    {templates:process(<span data-template="nav-utils:make-work-overview" />, $model)}
                  </ul>
                </li>
        }

        <li class="dropdown">
          <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button"
          aria-haspopup="true" aria-expanded="false">Legende <span class="caret"></span></a>
          <ul id="navbar-edition-burger-keys" class="dropdown-menu">
            {templates:process(<span data-template="nav-utils:make-key-content" />, $model)}
          </ul>
        </li>
        {
            if (not(local:in-split-xml())) then
                ()
            else
                <li class="dropdown">
                  <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button"
                  aria-haspopup="true" aria-expanded="false">Register <span class="caret"></span></a>
                  <ul class="dropdown-menu">
                    {
                    let $file := request:get-parameter("file", "")
                    return nav-utils:inject-indices($file)
                    }
                  </ul>
                </li>
        }
      </ul>
    </div>
  </div>
};

declare function local:in-split-xml() as xs:boolean
{
    let $file-param := request:get-parameter("file", "")
    return
        if (matches($file-param, "-")) then
            true()
        else
            false()
};


(:~ Inserts only the indices to the navbar that actually have at least an entry
 : in a given work.
 :
 : @author Michelle Weidling
 : @param $file The current file
 : @return Links to the occuring indices  :)
declare function nav-utils:inject-indices($file as xs:string) as element(xhtml:a)*
{
    let $whitelist := doc($config:aux || "whitelist.xml")
    let $indices := $whitelist//bdn:work[descendant::bdn:file[matches(., $file)]]/bdn:index/@type/string()
    let $mapping :=
        map {"bible-index": "Bibelstellen",
            "persons-index": "Personen",
            "subjects-index": "Sachen",
            "classics-index": "Antike Autoren"
        }

    return
        for $index in $indices return
            <a class="dropdown-item" href="{local:make-index-link($index)}">{map:get($mapping, $index)}</a>

};

declare function nav-utils:index-navbar-injection($node as node(),
$model as map(*), $navtype as xs:string) as element(nav)*
{
  let $index-type := request:get-parameter("type", "")
  let $mode := request:get-parameter("mode", "")
  let $file := request:get-parameter("file", "")

  let $nav-id :=
    if ($navtype = "selection") then
      "index-selection-nav"
    else
      "index-nav"
  let $div-id :=
    if ($navtype = "selection") then
      "index-selection-navbar"
    else
      "index-navbar"
  return
    if ($mode != "") then
      <nav class="navbar navbar-default" id="{$nav-id}">
        <div class="container">
          <div class="row">
            <div class="dropdown-container" id="{$div-id}">
              <ul class="nav navbar-nav">
              {
                nav-utils:make-navbar-content($navtype, $index-type, $file, $mode)
              }
              </ul>
            </div>
          </div>
        </div>
      </nav>
    else
      ()
};

declare function nav-utils:make-navbar-content($navtype as xs:string,
$index-type as xs:string, $file as xs:string, $mode as xs:string)
as element(li)*
{
    if ($navtype = "selection") then
      let $url := "index-overview.html?file=" || $file || "&amp;type=" ||
        $index-type || "&amp;mode=" || $mode
      let $type-label-map :=
        map
        {
          "bible-index": "Bibelregister",
          "persons-index": "Personenregister",
          "subjects-index": "Sachregister",
          "classics-index": "Register antiker Autoren"
        }
        let $whitelist := doc($config:aux || "whitelist.xml")
        let $indices := $whitelist//bdn:work[descendant::bdn:file[matches(., $file)]]/bdn:index/@type/string()

      return
          for $type in $indices
          order by $type
            return
                let $new-url := replace($url, $index-type, $type)
                let $class :=
                  if ($type = $index-type) then
                    "index-tag-link disabled"
                  else
                    "index-tag-link"
                return
                  <li>
                    <a href="{$new-url}" class="{$class}">
                      {$type-label-map?($type)}
                    </a>
                  </li>
    else
      if ($index-type = "bible-index") then
        (<li><a href="#bible-index" class="index-tag-link">Altes Testament</a></li>,
        <li><a href="#bible-index-nt" class="index-tag-link">Neues Testament</a></li>)
      else
        let $author := substring-before($file, "-")
        let $index-file := doc($config:aux || $author || "-" || $index-type || ".xml")
        let $letters :=
          for $letter in $index-file//bdn:letter return
            $letter/@id/string()
        for $letter in $letters return
          <li><a href="#{$letter}" class="index-tag-link">{$letter}</a></li>
};


declare function nav-utils:a-z-navigation() as element(nav)
{
  <nav class="navbar navbar-default" id="index-nav">
    <div class="container-fluid">
      <div class="collapse navbar-collapse" id="index-navbar">
        <ul class="nav navbar-nav">
        {
          for $iii in ("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K",
          "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X",
          "Y", "Z") return
            <li class="index-nav"><a href="#{$iii}">{$iii}</a></li>
        }
        </ul>
      </div>
    </div>
  </nav>
};

declare function local:make-index-link($type as xs:string) as xs:string
{
  let $html-prefix := "index-overview.html?"
  let $file-param := "file=" || request:get-parameter("file", "")
  let $index-type-prefix := "&amp;type="

  return
    $html-prefix || $file-param || $index-type-prefix || $type
};


declare function nav-utils:make-work-overview($node as node()*, $model as map(*))
as element(xhtml:a)*
{
    let $current-view := request:get-parameter("edition", "digital")
    let $filename := request:get-parameter("file", "")
    let $author :=
        if (matches($filename, "bahrdt-semler")) then
            "bahrdt-semler"
        else if (matches($filename, "-")) then
            substring-before($filename, "-")
        else
            ()
    let $author-entry :=
        if ($author) then
            whitelist:get-author-entry($author)
        else
            ()
    let $nav-entries :=
        if ($author != "") then
            $author-entry/following-sibling::bdn:nav-entries/bdn:entry
        else
            ()

    let $no-of-entries := count($nav-entries)
    return
        for $entry in $nav-entries
            let $link := "book.html?file=" || $entry/@source || "&amp;edition=" || $current-view
            return
                if ($entry/@omitting-printing = $current-view) then
                    ()
                else if ($entry/bdn:subentry) then
                        (
                            element xhtml:div
                            {
                                attribute class {"overview-entry dropdown-item"},
                                text{$entry/text()[1] || " "},
                                if ($entry/bdn:subentry/bdn:subentry) then
                                    ()
                                else
                                    <i class="fas fa-caret-down"></i>
                            },


                            if ($entry/bdn:subentry/bdn:subentry) then
                                for $subentry in $entry/bdn:subentry return
                                    if ($subentry/bdn:subentry) then
                                        (
                                            element xhtml:div
                                            {
                                                attribute class {"overview-entry dropdown-item overview-entry-level-2"},
                                                text{string-join($subentry/text()[1], " ") || " "},
                                                if ($subentry/bdn:subentry) then
                                                    <i class="fas fa-caret-down"></i>
                                                else
                                                    ()
                                            },

                                            nav-utils:make-work-overview-subentries($subentry, $current-view)
                                        )
                                    else
                                        let $link := "book.html?file=" || $subentry/@source/string() || "&amp;edition=" || $current-view
                                        return
                                            element xhtml:a
                                            {
                                                attribute href {$link},
                                                element xhtml:div
                                                {
                                                    attribute class {"overview-main-entry dropdown-item overview-entry-level-2"},
                                                    text{$subentry/text()[1] || " "}
                                                }
                                            }


                            else
                                nav-utils:make-work-overview-subentries($entry, $current-view),
                            if (not($entry/parent::*/child::*[last()] = $entry)) then
                                <div class="dropdown-divider" />
                            else
                                ()
                        )
                else
                    (
                        element xhtml:a
                        {
                            attribute href {$link},
                            element xhtml:div
                            {
                                attribute class {"overview-main-entry dropdown-item"},
                                text{$entry/text()[1] || " "}
                            }
                        },
                        if (not($entry/parent::*/child::*[last()] = $entry)) then
                            <div class="dropdown-divider" />
                        else
                            ()
                    )
};


declare function nav-utils:make-work-overview-subentries($entry as element(*),
$current-view as xs:string) as element(xhtml:div)
{
    element xhtml:div
    {
        attribute class {"overview-subentry-container"},
        for $subentry in $entry/bdn:subentry return
        let $file := $subentry/@source/string()
            let $subentry-link := "book.html?file="
                || $file
                || "&amp;edition="
                || $current-view
            return
                element xhtml:a
                {
                    attribute class {"overview-subentry dropdown-item"},
                    attribute href {$subentry-link},
                    text{$subentry/text()[1]}
                }
    }
};


declare function nav-utils:make-metadata($node as node()*, $model as map(*))
{
    let $filename := request:get-parameter("file", "")
        => concat(".xml")
    let $last-saved := doc($config:sade-meta || $filename)//tgmd:lastModified/string()
    let $last-modified := xmldb:last-modified($config:sade-data, $filename)


    return
        <div class="metadata">
            <div class="metadata-section"><strong>Zuletzt im TGLab gespeichert:</strong></div>
            <div>{$last-saved}</div>

            <div class="metadata-section"><strong>Zuletzt ins Portal übertragen:</strong></div>
            <div>{$last-modified}</div>
        </div>

};
